@@ -2,8 +2,7 @@
 
 import inspect
 import mock
-import sys
-from unittest import TestCase, skipIf
+from unittest import TestCase
 from nose.tools import assert_equal, assert_raises
 
 from .parameterized import (
@@ -242,7 +241,6 @@ class TestParamerizedOnTestCase(TestCase):
         missing_tests.remove("%s(%r, bar=%r)" %(expected_name, foo, bar))
 
 
-@skipIf(sys.version_info[:2] >= (3, 8), "Doesn't work with Python 3.8")
 class TestParameterizedExpandDocstring(TestCase):
     def _assert_docstring(self, expected_docstring, rstrip=False):
         """ Checks the current test method's docstring. Must be called directly
@@ -251,7 +249,10 @@ class TestParameterizedExpandDocstring(TestCase):
         f_locals = stack[3][0].f_locals
         test_method = (
             f_locals.get("testMethod") or # Py27
-            f_locals.get("function") # Py33
+            f_locals.get("function") or # Py33
+            f_locals.get("method") or # Py38
+            f_locals.get("testfunction") or # Py382
+            None
         )
         if test_method is None:
             raise AssertionError("uh oh, unittest changed a local variable name")
@@ -446,17 +447,14 @@ def test_cases_over_10(input, expected):
 
 @parameterized_class(("a", "b", "c"), [
     ("foo", 1, 2),
-    ("bar", 3, 0),
     (0, 1, 2),
 ])
 class TestParameterizedClass(TestCase):
     expect([
         "TestParameterizedClass_0_foo:test_method_a('foo', 1, 2)",
         "TestParameterizedClass_0_foo:test_method_b('foo', 1, 2)",
-        "TestParameterizedClass_1_bar:test_method_a('bar', 3, 0)",
-        "TestParameterizedClass_1_bar:test_method_b('bar', 3, 0)",
-        "TestParameterizedClass_2:test_method_a(0, 1, 2)",
-        "TestParameterizedClass_2:test_method_b(0, 1, 2)",
+        "TestParameterizedClass_1:test_method_a(0, 1, 2)",
+        "TestParameterizedClass_1:test_method_b(0, 1, 2)",
     ])
 
     def _assertions(self, test_name):
@@ -477,22 +475,43 @@ class TestParameterizedClass(TestCase):
         self._assertions("test_method_b")
 
 
+@parameterized_class(("a", ), [
+    (1, ),
+    (2, ),
+], class_name_func=lambda cls, idx, attrs: "%s_custom_func_%s" %(cls.__name__, attrs["a"]))
+class TestNamedParameterizedClass(TestCase):
+    expect([
+        "TestNamedParameterizedClass_custom_func_1:test_method(1)",
+        "TestNamedParameterizedClass_custom_func_2:test_method(2)",
+    ])
+
+    def test_method(self):
+        missing_tests.remove("%s:test_method(%r)" %(
+            self.__class__.__name__,
+            self.a,
+        ))
+
+
 @parameterized_class([
-    {"foo": 1},
-    {"bar": 1},
+    {"foo": 42},
+    {"bar": "some stuff"},
+    {"bar": "other stuff", "name": "some name", "foo": 12},
 ])
 class TestParameterizedClassDict(TestCase):
     expect([
-        "TestParameterizedClassDict_0:setUp(1, 0)",
-        "TestParameterizedClassDict_0:tearDown(1, 0)",
-        "TestParameterizedClassDict_0:test_method(1, 0)",
-        "TestParameterizedClassDict_1:test_method(0, 1)",
-        "TestParameterizedClassDict_1:setUp(0, 1)",
-        "TestParameterizedClassDict_1:tearDown(0, 1)",
+        "TestParameterizedClassDict_0:setUp(42, 'empty')",
+        "TestParameterizedClassDict_0:test_method(42, 'empty')",
+        "TestParameterizedClassDict_0:tearDown(42, 'empty')",
+        "TestParameterizedClassDict_1_some_stuff:setUp(0, 'some stuff')",
+        "TestParameterizedClassDict_1_some_stuff:test_method(0, 'some stuff')",
+        "TestParameterizedClassDict_1_some_stuff:tearDown(0, 'some stuff')",
+        "TestParameterizedClassDict_2_some_name:setUp(12, 'other stuff')",
+        "TestParameterizedClassDict_2_some_name:test_method(12, 'other stuff')",
+        "TestParameterizedClassDict_2_some_name:tearDown(12, 'other stuff')",
     ])
 
     foo = 0
-    bar = 0
+    bar = 'empty'
 
     def setUp(self):
         # Ensure that super() works (issue #73)
